@@ -47,7 +47,7 @@ SSHConfig = collections.namedtuple(
 
 ChecksConfig = collections.namedtuple(
     'ChecksConfig',
-    ['health_check', 'query', 'expected_rows', 'expected_result']
+    ['health_check', 'query', 'expected_rows', 'expected_result', 'enable_md5_checks']
 )
 
 MonitoringConfig = collections.namedtuple(
@@ -72,7 +72,7 @@ GrpcConfig = collections.namedtuple(
 
 KubernetesConfig = collections.namedtuple(
     'KubernetesConfig',
-    ['enabled', 'cassandra_url']
+    ['enabled', 'cassandra_url', 'use_mgmt_api']
 )
 
 DEFAULT_CONFIGURATION_PATH = pathlib.Path('/etc/medusa/medusa.ini')
@@ -126,7 +126,8 @@ def load_config(args, config_file):
         'health_check': 'cql',
         'query': '',
         'expected_rows': '0',
-        'expected_result': ''
+        'expected_result': '',
+        'enable_md5_checks': 'false'
     }
 
     config['monitoring'] = {
@@ -139,7 +140,8 @@ def load_config(args, config_file):
 
     config['kubernetes'] = {
         'enabled': False,
-        'cassandra_url': 'None'
+        'cassandra_url': 'None',
+        'use_mgmt_api': False
     }
 
     if config_file:
