@@ -115,19 +115,27 @@ def cli(ctx, verbosity, without_log_timestamp, config_file, **kwargs):
 @cli.command(aliases=['backup', 'backup-node'])
 @click.option('--backup-name', help='Custom name for the backup')
 @click.option('--stagger', default=None, type=int, help='Drop initial backups if longer than a duration in seconds')
+@click.option('--enable-md5-checks',
+              help='During backups and verify, use md5 calculations to determine file integrity '
+                   '(in addition to size, which is used by default)',
+              is_flag=True, default=False)
 @click.option('--mode', default="differential", type=click.Choice(['full', 'differential']))
 @pass_MedusaConfig
-def backup(medusaconfig, backup_name, stagger, mode):
+def backup(medusaconfig, backup_name, stagger, enable_md5_checks, mode):
     """
     Backup single Cassandra node
     """
     stagger_time = datetime.timedelta(seconds=stagger) if stagger else None
-    medusa.backup_node.main(medusaconfig, backup_name, stagger_time, mode)
+    medusa.backup_node.main(medusaconfig, backup_name, stagger_time, enable_md5_checks, mode)
 
 
 @cli.command(name='backup-cluster')
 @click.option('--backup-name', help='Backup name', required=True)
 @click.option('--stagger', default=None, type=int, help='Drop initial backups if longer than a duration in seconds')
+@click.option('--enable-md5-checks',
+              help='During backups and verify, use md5 calculations to determine file integrity '
+                   '(in addition to size, which is used by default)',
+              is_flag=True, default=False)
 @click.option('--mode', default="differential", type=click.Choice(['full', 'differential']))
 @click.option('--temp-dir', help='Directory for temporary storage', default="/tmp")
 @click.option('--parallel-snapshots', '-ps', help="Number of concurrent synchronous (blocking) "
@@ -135,13 +143,15 @@ def backup(medusaconfig, backup_name, stagger, mode):
 @click.option('--parallel-uploads', '-pu', help="Number of concurrent synchronous (blocking) "
                                                 "ssh sessions started by pssh", default=1)
 @pass_MedusaConfig
-def backup_cluster(medusaconfig, backup_name, stagger, mode, temp_dir, parallel_snapshots, parallel_uploads):
+def backup_cluster(medusaconfig, backup_name, stagger, enable_md5_checks, mode, temp_dir,
+                   parallel_snapshots, parallel_uploads):
     """
     Backup Cassandra cluster
     """
     medusa.backup_cluster.orchestrate(medusaconfig,
                                       backup_name,
                                       stagger,
+                                      enable_md5_checks,
                                       mode,
                                       Path(temp_dir),
                                       int(parallel_snapshots),
@@ -263,12 +273,15 @@ def status(medusaconfig, backup_name):
 
 @cli.command(name='verify')
 @click.option('--backup-name', help='Backup name', required=True)
+@click.option('--enable-md5-checks', help='During backups and verify, use md5 calculations to determine file integrity '
+                                          '(in addition to size, which is used by default)',
+              is_flag=True, default=False)
 @pass_MedusaConfig
-def verify(medusaconfig, backup_name):
+def verify(medusaconfig, backup_name, enable_md5_checks):
     """
     Verify the integrity of a backup
     """
-    medusa.verify.verify(medusaconfig, backup_name)
+    medusa.verify.verify(medusaconfig, backup_name, enable_md5_checks)
 
 
 @cli.command(name='report-last-backup')
