@@ -19,6 +19,7 @@ import datetime
 import json
 import logging
 import os
+import requests
 import shutil
 import subprocess
 import time
@@ -29,17 +30,22 @@ from pathlib import Path
 from subprocess import PIPE
 import signal
 from cassandra import ProtocolVersion
+import cassandra.cluster
 from cassandra.cluster import Cluster
 from ssl import SSLContext, PROTOCOL_TLS, PROTOCOL_TLSv1, PROTOCOL_TLSv1_1, PROTOCOL_TLSv1_2, CERT_REQUIRED
+from zipfile import ZipFile
 
 import medusa.backup_node
+import medusa.download
 import medusa.index
+import medusa.filtering
 import medusa.listing
 import medusa.purge
 import medusa.report_latest
 import medusa.restore_node
 import medusa.status
 import medusa.verify
+import medusa.verify_restore
 import medusa.service.grpc.client
 
 from medusa.config import (
@@ -62,15 +68,25 @@ usercert = "{}/resources/local_with_ssl/client.pem".format(os.getcwd())
 userkey = "{}/resources/local_with_ssl/client.key.pem".format(os.getcwd())
 keystore_path = "{}/resources/local_with_ssl/127.0.0.1.jks".format(os.getcwd())
 trustore_path = "{}/resources/local_with_ssl/generic-server-truststore.jks".format(os.getcwd())
+config_checks = {"health_check": "cql", "enable_md5_checks": "false"}
+NOT_A_VALID_CLUSTER_MSG = b"does not appear to be a valid cluster"
+STARTING_TESTS_MSG = "Starting the tests"
+CCM_STOP = "ccm stop"
+CCM_START = "ccm start"
+CCM_DIR = "~/.ccm"
+BUCKET_ROOT = "/tmp/medusa_it_bucket"
+CASSANDRA_YAML = "cassandra.yaml"
+AWS_CREDENTIALS = "~/.aws/credentials"
+GCS_CREDENTIALS = "~/medusa_credentials.json"
 
 
 def kill_cassandra():
-    p = subprocess.Popen(["ps", "-A"], stdout=subprocess.PIPE)
+    p = subprocess.Popen(["ps", "-Af"], stdout=subprocess.PIPE)
     out, err = p.communicate()
     for line in out.splitlines():
         if b"org.apache.cassandra.service.CassandraDaemon" in line:
             logging.info(line)
-            pid = int(line.split(None, 1)[0])
+            pid = int(line.split(None, 2)[1])
             os.kill(pid, signal.SIGKILL)
 
 
@@ -90,6 +106,28 @@ def cleanup_storage(context, storage_provider):
             storage.storage_driver.delete_object(obj)
 
 
+def get_client_encryption_opts(keystore_path, trustore_path):
+    return f"""ccm node1 updateconf -y 'client_encryption_options: {{ enabled: true,
+        optional: false,keystore: {keystore_path}, keystore_password: testdata1,
+        require_client_auth: true,truststore: {trustore_path},  truststore_password: truststorePass1,
+        protocol: TLS,algorithm: SunX509,store_type: JKS,cipher_suites: [TLS_RSA_WITH_AES_256_CBC_SHA]}}'"""
+
+
+def tune_ccm_settings(cluster_name):
+    if os.uname().sysname == "Linux":
+        os.popen(
+            """sed -i 's/#MAX_HEAP_SIZE="4G"/MAX_HEAP_SIZE="256m"/' ~/.ccm/"""
+            + cluster_name
+            + """/node1/conf/cassandra-env.sh"""
+        ).read()
+        os.popen(
+            """sed -i 's/#HEAP_NEWSIZE="800M"/HEAP_NEWSIZE="200M"/' ~/.ccm/"""
+            + cluster_name
+            + """/node1/conf/cassandra-env.sh"""
+        ).read()
+    os.popen("LOCAL_JMX=yes ccm start --no-wait").read()
+
+
 class GRPCServer:
     @staticmethod
     def init(config):
@@ -123,6 +161,56 @@ class GRPCServer:
             subprocess.Popen(cmd, cwd=os.path.abspath("../"))
 
 
+class MgmtApiServer:
+    @staticmethod
+    def init(config, cluster_name):
+        server = MgmtApiServer(config, cluster_name)
+        server.start()
+        return server
+
+    @staticmethod
+    def destroy():
+        p = subprocess.Popen(["ps", "-Af"], stdout=subprocess.PIPE)
+        out, err = p.communicate()
+        for line in out.splitlines():
+            if b"mgmtapi.sock" in line:
+                logging.info(line)
+                pid = int(line.split(None, 2)[1])
+                os.kill(pid, signal.SIGKILL)
+
+    def __init__(self, config, cluster_name):
+        self.cluster_name = cluster_name
+        env = {**os.environ, "MGMT_API_LOG_DIR": '/tmp'}
+        cmd = ["java", "-jar", "/tmp/management-api-server/target/datastax-mgmtapi-server-0.1.0-SNAPSHOT.jar",
+               "--db-socket=/tmp/db.sock",
+               "--host=unix:///tmp/mgmtapi.sock",
+               "--host=http://localhost:8080",
+               "--db-home={}/.ccm/{}/node1".format(str(Path.home()), self.cluster_name),
+               "--explicit-start",
+               "true",
+               "--no-keep-alive",
+               "true",
+               ]
+        subprocess.Popen(cmd, cwd=os.path.abspath("../"), env=env)
+
+    @staticmethod
+    def start():
+        started = False
+        start_count = 0
+        while not started and start_count < 20:
+            try:
+                start_count += 1
+                requests.post("http://127.0.0.1:8080/api/v0/lifecycle/start")
+                started = True
+            except Exception:
+                # wait for Cassandra to start
+                time.sleep(1)
+
+    @staticmethod
+    def stop():
+        requests.post("http://127.0.0.1:8080/api/v0/lifecycle/stop")
+
+
 @given(r'I have a fresh ccm cluster "{client_encryption}" running named "{cluster_name}"')
 def _i_have_a_fresh_ccm_cluster_running(context, cluster_name, client_encryption):
     context.session = None
@@ -133,7 +221,7 @@ def _i_have_a_fresh_ccm_cluster_running(context, cluster_name, client_encryption
     res = subprocess.run(
         ["ccm", "switch", context.cluster_name], stdout=PIPE, stderr=PIPE
     )
-    if b"does not appear to be a valid cluster" not in res.stderr:
+    if NOT_A_VALID_CLUSTER_MSG not in res.stderr:
         subprocess.check_call(
             ["ccm", "remove", context.cluster_name], stdout=PIPE, stderr=PIPE
         )
@@ -151,24 +239,10 @@ def _i_have_a_fresh_ccm_cluster_running(context, cluster_name, client_encryption
 
     if client_encryption == 'with_client_encryption':
         is_client_encryption_enable = True
-        update_client_encrytion_opts = "ccm node1 updateconf -y 'client_encryption_options: { enabled: true,\
-        optional: false,keystore: " + keystore_path + ",keystore_password: testdata1,\
-        require_client_auth: true,truststore: " + trustore_path + ",truststore_password: truststorePass1,\
-        protocol: TLS,algorithm: SunX509,store_type: JKS,cipher_suites: [TLS_RSA_WITH_AES_256_CBC_SHA]}'"
+        update_client_encrytion_opts = get_client_encryption_opts(keystore_path, trustore_path)
         os.popen(update_client_encrytion_opts).read()
 
-    if os.uname().sysname == "Linux":
-        os.popen(
-            """sed -i 's/#MAX_HEAP_SIZE="4G"/MAX_HEAP_SIZE="256m"/' ~/.ccm/"""
-            + context.cluster_name
-            + """/node1/conf/cassandra-env.sh"""
-        ).read()
-        os.popen(
-            """sed -i 's/#HEAP_NEWSIZE="800M"/HEAP_NEWSIZE="200M"/' ~/.ccm/"""
-            + context.cluster_name
-            + """/node1/conf/cassandra-env.sh"""
-        ).read()
-    os.popen("LOCAL_JMX=yes ccm start --no-wait").read()
+    tune_ccm_settings(context.cluster_name)
     context.session = connect_cassandra(is_client_encryption_enable)
 
 
@@ -183,7 +257,7 @@ def _i_have_a_fresh_ccm_cluster_with_jolokia_running(context, cluster_name, clie
     res = subprocess.run(
         ["ccm", "switch", context.cluster_name], stdout=PIPE, stderr=PIPE
     )
-    if b"does not appear to be a valid cluster" not in res.stderr:
+    if NOT_A_VALID_CLUSTER_MSG not in res.stderr:
         subprocess.check_call(
             ["ccm", "remove", context.cluster_name], stdout=PIPE, stderr=PIPE
         )
@@ -201,10 +275,7 @@ def _i_have_a_fresh_ccm_cluster_with_jolokia_running(context, cluster_name, clie
 
     if client_encryption == 'with_client_encryption':
         is_client_encryption_enable = True
-        update_client_encrytion_opts = "ccm node1 updateconf -y 'client_encryption_options: { enabled: true,\
-        optional: false,keystore: " + keystore_path + ",keystore_password: testdata1,\
-        require_client_auth: true,truststore: " + trustore_path + ",truststore_password: truststorePass1,\
-        protocol: TLS,algorithm: SunX509,store_type: JKS,cipher_suites: [TLS_RSA_WITH_AES_256_CBC_SHA]}'"
+        update_client_encrytion_opts = get_client_encryption_opts(keystore_path, trustore_path)
         os.popen(update_client_encrytion_opts).read()
 
     conf_file = os.path.expanduser("~/.ccm/{}/node1/conf/cassandra-env.sh".format(context.cluster_name))
@@ -214,24 +285,58 @@ def _i_have_a_fresh_ccm_cluster_with_jolokia_running(context, cluster_name, clie
         )
     shutil.copyfile("resources/grpc/jolokia-jvm-1.6.2-agent.jar", "/tmp/jolokia-jvm-1.6.2-agent.jar")
 
-    if os.uname().sysname == "Linux":
-        os.popen(
-            """sed -i 's/#MAX_HEAP_SIZE="4G"/MAX_HEAP_SIZE="256m"/' ~/.ccm/"""
-            + context.cluster_name
-            + """/node1/conf/cassandra-env.sh"""
-        ).read()
-        os.popen(
-            """sed -i 's/#HEAP_NEWSIZE="800M"/HEAP_NEWSIZE="200M"/' ~/.ccm/"""
-            + context.cluster_name
-            + """/node1/conf/cassandra-env.sh"""
-        ).read()
-    os.popen("LOCAL_JMX=yes ccm start --no-wait").read()
+    tune_ccm_settings(context.cluster_name)
     context.session = connect_cassandra(is_client_encryption_enable)
 
 
+@given(r'I have a fresh ccm cluster with mgmt api "{client_encryption}" named "{cluster_name}"')
+def _i_have_a_fresh_ccm_cluster_with_mgmt_api_running(context, cluster_name, client_encryption):
+    context.session = None
+    context.cluster_name = cluster_name
+    is_client_encryption_enable = False
+    subprocess.run(["ccm", "stop"], stdout=PIPE, stderr=PIPE)
+    kill_cassandra()
+    res = subprocess.run(
+        ["ccm", "switch", context.cluster_name], stdout=PIPE, stderr=PIPE
+    )
+    if NOT_A_VALID_CLUSTER_MSG not in res.stderr:
+        subprocess.check_call(
+            ["ccm", "remove", context.cluster_name], stdout=PIPE, stderr=PIPE
+        )
+    subprocess.check_call(
+        [
+            "ccm",
+            "create",
+            context.cluster_name,
+            "-v",
+            context.cassandra_version,
+            "-n",
+            "1",
+        ]
+    )
+
+    if client_encryption == 'with_client_encryption':
+        is_client_encryption_enable = True
+        update_client_encrytion_opts = get_client_encryption_opts(keystore_path, trustore_path)
+        os.popen(update_client_encrytion_opts).read()
+
+    tune_ccm_settings(context.cluster_name)
+    context.session = connect_cassandra(is_client_encryption_enable)
+    # stop the node via CCM as it needs to be started by the Management API
+    os.popen(CCM_STOP).read()
+
+    conf_file = os.path.expanduser("~/.ccm/{}/node1/conf/cassandra-env.sh".format(context.cluster_name))
+    with open(conf_file, "a") as config_file:
+        config_file.write(
+            'JVM_OPTS="$JVM_OPTS -javaagent:/tmp/management-api-agent/target/datastax-mgmtapi-agent-0.1.0-SNAPSHOT.jar"'
+        )
+    # get the Cassandra Management API jars
+    get_mgmt_api_jars(version=context.cassandra_version)
+
+
 @given(r'I am using "{storage_provider}" as storage provider in ccm cluster "{client_encryption}"')
 def i_am_using_storage_provider(context, storage_provider, client_encryption):
-    logging.info("Starting the tests")
+    logging.info(STARTING_TESTS_MSG)
     if not hasattr(context, "cluster_name"):
         context.cluster_name = "test"
     config = configparser.ConfigParser(interpolation=None)
@@ -256,7 +361,7 @@ def i_am_using_storage_provider(context, storage_provider, client_encryption):
         config["storage"] = {
             "host_file_separator": ",",
             "bucket_name": "medusa-integration-tests",
-            "key_file": "~/medusa_credentials.json",
+            "key_file": GCS_CREDENTIALS,
             "storage_provider": "google_storage",
             "fqdn": "127.0.0.1",
             "api_key_or_username": "",
@@ -278,11 +383,11 @@ def i_am_using_storage_provider(context, storage_provider, client_encryption):
             "multi_part_upload_threshold": 1 * 1024,
             "prefix": storage_prefix
         }
-    elif storage_provider.startswith("s3"):
+    elif storage_provider == "s3_us_west_oregon":
         config["storage"] = {
             "host_file_separator": ",",
             "bucket_name": "tlp-medusa-dev",
-            "key_file": "~/.aws/credentials",
+            "key_file": AWS_CREDENTIALS,
             "storage_provider": storage_provider,
             "fqdn": "127.0.0.1",
             "api_key_or_username": "",
@@ -294,6 +399,23 @@ def i_am_using_storage_provider(context, storage_provider, client_encryption):
             "prefix": storage_prefix,
             "aws_cli_path": "aws"
         }
+    elif storage_provider == "minio":
+        config["storage"] = {
+            "host_file_separator": ",",
+            "bucket_name": "medusa-dev",
+            "key_file": "~/.aws/minio_credentials",
+            "storage_provider": "s3_compatible",
+            "fqdn": "127.0.0.1",
+            "api_profile": "default",
+            "base_path": "/tmp",
+            "multi_part_upload_threshold": 1 * 1024,
+            "concurrent_transfers": 1,
+            "prefix": storage_prefix,
+            "aws_cli_path": "aws",
+            "host": "localhost",
+            "port": 9000,
+            "secure": False
+        }
     elif storage_provider.startswith("ibm"):
         config["storage"] = {
             "host_file_separator": ",",
@@ -316,18 +438,18 @@ def i_am_using_storage_provider(context, storage_provider, client_encryption):
 
     config["cassandra"] = {
         "is_ccm": 1,
-        "stop_cmd": "ccm stop",
-        "start_cmd": "ccm start",
+        "stop_cmd": CCM_STOP,
+        "start_cmd": CCM_START,
         "cql_username": "cassandra",
         "cql_password": "cassandra",
         "config_file": os.path.expanduser(
             os.path.join(
-                "~/.ccm", context.cluster_name, "node1", "conf", "cassandra.yaml"
+                CCM_DIR, context.cluster_name, "node1", "conf", CASSANDRA_YAML
             )
         ),
         "sstableloader_bin": os.path.expanduser(
             os.path.join(
-                "~/.ccm",
+                CCM_DIR,
                 "repository",
                 context.cassandra_version.replace(
                     "github:", "githubCOLON").replace("/", "SLASH"),
@@ -353,9 +475,7 @@ def i_am_using_storage_provider(context, storage_provider, client_encryption):
 
     config["monitoring"] = {"monitoring_provider": "local"}
 
-    config["checks"] = {
-        "health_check": "cql"
-    }
+    config["checks"] = config_checks
 
     config["grpc"] = {
         "enabled": "0"
@@ -381,106 +501,7 @@ def i_am_using_storage_provider(context, storage_provider, client_encryption):
 
 @given(r'I am using "{storage_provider}" as storage provider in ccm cluster "{client_encryption}" with gRPC server')
 def i_am_using_storage_provider_with_grpc_server(context, storage_provider, client_encryption):
-    logging.info("Starting the tests")
-    if not hasattr(context, "cluster_name"):
-        context.cluster_name = "test"
-    config = configparser.ConfigParser(interpolation=None)
-
-    if storage_provider == "local":
-        if os.path.isdir(os.path.join("/tmp", "medusa_it_bucket")):
-            shutil.rmtree(os.path.join("/tmp", "medusa_it_bucket"))
-        os.makedirs(os.path.join("/tmp", "medusa_it_bucket"))
-
-        config["storage"] = {
-            "host_file_separator": ",",
-            "bucket_name": "medusa_it_bucket",
-            "key_file": "",
-            "storage_provider": "local",
-            "fqdn": "127.0.0.1",
-            "api_key_or_username": "",
-            "api_secret_or_password": "",
-            "base_path": "/tmp",
-            "prefix": storage_prefix
-        }
-    elif storage_provider == "google_storage":
-        config["storage"] = {
-            "host_file_separator": ",",
-            "bucket_name": "medusa-integration-tests",
-            "key_file": "~/medusa_credentials.json",
-            "storage_provider": "google_storage",
-            "fqdn": "127.0.0.1",
-            "api_key_or_username": "",
-            "api_secret_or_password": "",
-            "base_path": "/tmp",
-            "prefix": storage_prefix
-        }
-    elif storage_provider.startswith("s3"):
-        config["storage"] = {
-            "host_file_separator": ",",
-            "bucket_name": "tlp-medusa-dev",
-            "key_file": "~/.aws/credentials",
-            "storage_provider": storage_provider,
-            "fqdn": "127.0.0.1",
-            "api_key_or_username": "",
-            "api_secret_or_password": "",
-            "api_profile": "default",
-            "base_path": "/tmp",
-            "multi_part_upload_threshold": 1 * 1024,
-            "concurrent_transfers": 4,
-            "prefix": storage_prefix,
-            "aws_cli_path": "aws"
-        }
-
-    config["cassandra"] = {
-        "is_ccm": 1,
-        "stop_cmd": "ccm stop",
-        "start_cmd": "ccm start",
-        "cql_username": "cassandra",
-        "cql_password": "cassandra",
-        "config_file": os.path.expanduser(
-            os.path.join(
-                "~/.ccm", context.cluster_name, "node1", "conf", "cassandra.yaml"
-            )
-        ),
-        "sstableloader_bin": os.path.expanduser(
-            os.path.join(
-                "~/.ccm",
-                "repository",
-                context.cassandra_version,
-                "bin",
-                "sstableloader",
-            )
-        ),
-        "resolve_ip_addresses": False
-    }
-
-    if client_encryption == 'with_client_encryption':
-        config["cassandra"].update(
-            {
-                "certfile": certfile,
-                "usercert": usercert,
-                "userkey": userkey,
-                "sstableloader_ts": trustore_path,
-                "sstableloader_tspw": "truststorePass1",
-                "sstableloader_ks": keystore_path,
-                "sstableloader_kspw": "testdata1"
-            }
-        )
-
-    config["monitoring"] = {"monitoring_provider": "local"}
-
-    config["checks"] = {
-        "health_check": "cql"
-    }
-
-    config["grpc"] = {
-        "enabled": 1,
-    }
-
-    config['kubernetes'] = {
-        "enabled": 1,
-        "cassandra_url": "http://127.0.0.1:8778/jolokia/",
-    }
+    config = get_medusa_config(context, storage_provider, client_encryption, "http://127.0.0.1:8778/jolokia/")
 
     GRPCServer.destroy()
     context.grpc_server = GRPCServer.init(config)
@@ -507,6 +528,153 @@ def i_am_using_storage_provider_with_grpc_server(context, storage_provider, clie
     time.sleep(3)
 
 
+@given(r'I am using "{storage_provider}" as storage provider in ccm cluster "{client_encryption}" with mgmt api')
+def i_am_using_storage_provider_with_grpc_server_and_mgmt_api(context, storage_provider, client_encryption):
+    config = get_medusa_config(context, storage_provider, client_encryption,
+                               "http://127.0.0.1:8080/api/v0/ops/node/snapshots", use_mgmt_api=1)
+
+    GRPCServer.destroy()
+    context.grpc_server = GRPCServer.init(config)
+
+    context.grpc_client = medusa.service.grpc.client.Client(
+        "127.0.0.1:50051",
+        channel_options=[('grpc.enable_retries', 0)]
+    )
+
+    MgmtApiServer.destroy()
+    context.mgmt_api_server = MgmtApiServer.init(config, context.cluster_name)
+
+    context.medusa_config = MedusaConfig(
+        storage=_namedtuple_from_dict(StorageConfig, config["storage"]),
+        cassandra=_namedtuple_from_dict(CassandraConfig, config["cassandra"]),
+        monitoring=_namedtuple_from_dict(MonitoringConfig, config["monitoring"]),
+        ssh=None,
+        checks=_namedtuple_from_dict(ChecksConfig, config["checks"]),
+        logging=None,
+        grpc=_namedtuple_from_dict(GrpcConfig, config["grpc"]),
+        kubernetes=_namedtuple_from_dict(KubernetesConfig, config['kubernetes']),
+    )
+
+    cleanup_storage(context, storage_provider)
+
+    is_client_encryption_enable = False
+    if client_encryption == 'with_client_encryption':
+        is_client_encryption_enable = True
+
+    # sleep for a few seconds to give gRPC server a chance to initialize
+    ready_count = 0
+    while ready_count < 20:
+        ready = 0
+        try:
+            ready = requests.get("http://127.0.0.1:8080/api/v0/probes/readiness").status_code
+        except Exception:
+            # wait for the server to be ready
+            time.sleep(1)
+        ready_count += 1
+        if ready == 200:
+            # server is ready, re-establish the session
+            context.session = connect_cassandra(is_client_encryption_enable)
+            break
+        else:
+            # wait for Cassandra to be ready
+            time.sleep(1)
+
+
+def get_medusa_config(context, storage_provider, client_encryption, cassandra_url, use_mgmt_api=0):
+    logging.info(STARTING_TESTS_MSG)
+    if not hasattr(context, "cluster_name"):
+        context.cluster_name = "test"
+    config = configparser.ConfigParser(interpolation=None)
+    if storage_provider == "local":
+        if os.path.isdir(os.path.join("/tmp", "medusa_it_bucket")):
+            shutil.rmtree(os.path.join("/tmp", "medusa_it_bucket"))
+        os.makedirs(os.path.join("/tmp", "medusa_it_bucket"))
+        config["storage"] = {
+            "host_file_separator": ",",
+            "bucket_name": "medusa_it_bucket",
+            "key_file": "",
+            "storage_provider": "local",
+            "fqdn": "127.0.0.1",
+            "api_key_or_username": "",
+            "api_secret_or_password": "",
+            "base_path": "/tmp",
+            "prefix": storage_prefix
+        }
+    elif storage_provider == "google_storage":
+        config["storage"] = {
+            "host_file_separator": ",",
+            "bucket_name": "medusa-integration-tests",
+            "key_file": GCS_CREDENTIALS,
+            "storage_provider": "google_storage",
+            "fqdn": "127.0.0.1",
+            "api_key_or_username": "",
+            "api_secret_or_password": "",
+            "base_path": "/tmp",
+            "prefix": storage_prefix
+        }
+    elif storage_provider.startswith("s3"):
+        config["storage"] = {
+            "host_file_separator": ",",
+            "bucket_name": "tlp-medusa-dev",
+            "key_file": AWS_CREDENTIALS,
+            "storage_provider": storage_provider,
+            "fqdn": "127.0.0.1",
+            "api_key_or_username": "",
+            "api_secret_or_password": "",
+            "api_profile": "default",
+            "base_path": "/tmp",
+            "multi_part_upload_threshold": 1 * 1024,
+            "concurrent_transfers": 4,
+            "prefix": storage_prefix,
+            "aws_cli_path": "aws"
+        }
+    config["cassandra"] = {
+        "is_ccm": 1,
+        "stop_cmd": CCM_STOP,
+        "start_cmd": CCM_START,
+        "cql_username": "cassandra",
+        "cql_password": "cassandra",
+        "config_file": os.path.expanduser(
+            os.path.join(
+                CCM_DIR, context.cluster_name, "node1", "conf", CASSANDRA_YAML
+            )
+        ),
+        "sstableloader_bin": os.path.expanduser(
+            os.path.join(
+                CCM_DIR,
+                "repository",
+                context.cassandra_version,
+                "bin",
+                "sstableloader",
+            )
+        ),
+        "resolve_ip_addresses": False
+    }
+    if client_encryption == 'with_client_encryption':
+        config["cassandra"].update(
+            {
+                "certfile": certfile,
+                "usercert": usercert,
+                "userkey": userkey,
+                "sstableloader_ts": trustore_path,
+                "sstableloader_tspw": "truststorePass1",
+                "sstableloader_ks": keystore_path,
+                "sstableloader_kspw": "testdata1"
+            }
+        )
+    config["monitoring"] = {"monitoring_provider": "local"}
+    config["checks"] = config_checks
+    config["grpc"] = {
+        "enabled": "1",
+    }
+    config['kubernetes'] = {
+        "enabled": 1,
+        "cassandra_url": cassandra_url,
+        "use_mgmt_api": use_mgmt_api,
+    }
+    return config
+
+
 @when(r'I create the "{table_name}" table in keyspace "{keyspace_name}"')
 def _i_create_the_whatever_table(context, table_name, keyspace_name):
     keyspace = """CREATE KEYSPACE IF NOT EXISTS {} WITH replication = {{'class':'SimpleStrategy',
@@ -543,10 +711,12 @@ def _i_run_a_whatever_command(context, command):
     os.popen(command).read()
 
 
-@when(r'I perform a backup in "{backup_mode}" mode of the node named "{backup_name}"')
-def _i_perform_a_backup_of_the_node_named_backupname(context, backup_mode, backup_name):
+@when(r'I perform a backup in "{backup_mode}" mode of the node named'
+      r' "{backup_name}" with md5 checks "{md5_enabled_str}"')
+def _i_perform_a_backup_of_the_node_named_backupname(context, backup_mode, backup_name, md5_enabled_str):
     (actual_backup_duration, actual_start, end, node_backup, node_backup_cache, num_files, start) \
-        = medusa.backup_node.main(context.medusa_config, backup_name, None, backup_mode)
+        = medusa.backup_node.main(context.medusa_config, backup_name, None,
+                                  str(md5_enabled_str).lower() == "enabled", backup_mode)
     context.latest_backup_cache = node_backup_cache
 
 
@@ -607,6 +777,12 @@ def _i_shutdown_the_grpc_server(context):
     context.grpc_server.destroy()
 
 
+@then(r'I shutdown the mgmt api server')
+def _i_shutdown_the_mgmt_api_server(context):
+    context.mgmt_api_server.stop()
+    context.mgmt_api_server.destroy()
+
+
 @then(r'I can see the backup named "{backup_name}" when I list the backups')
 def _i_can_see_the_backup_named_backupname_when_i_list_the_backups(
     context, backup_name
@@ -701,9 +877,9 @@ def _the_backup_named_backupname_has_nb_sstables_for_the_whatever_table(
         assert len(sstables) == int(nb_sstables)
 
 
-@then(r'I can verify the backup named "{backup_name}" successfully')
-def _i_can_verify_the_backup_named_successfully(context, backup_name):
-    medusa.verify.verify(context.medusa_config, backup_name)
+@then(r'I can verify the backup named "{backup_name}" with md5 checks "{md5_enabled_str}" successfully')
+def _i_can_verify_the_backup_named_successfully(context, backup_name, md5_enabled_str):
+    medusa.verify.verify(context.medusa_config, backup_name, str(md5_enabled_str).lower() == "enabled")
 
 
 @then(r'I can download the backup named "{backup_name}" for all tables')
@@ -826,7 +1002,7 @@ def _i_have_rows_in_the_table(context, nb_rows, table_name, client_encryption):
         is_client_encryption_enable = True
     context.session = connect_cassandra(is_client_encryption_enable)
     rows = context.session.execute("select count(*) as nb from {}".format(table_name))
-    assert rows[0].nb == int(nb_rows)
+    assert rows[0][0] == int(nb_rows)
 
 
 @then(r'I can see the backup index entry for "{backup_name}"')
@@ -879,7 +1055,7 @@ def _there_is_no_latest_backup_for_node_fqdn(context, fqdn):
 )
 def _node_fakes_a_complete_backup(context, fqdn, backup_name, backup_datetime):
     storage = Storage(config=context.medusa_config.storage)
-    path_root = "/tmp/medusa_it_bucket"
+    path_root = BUCKET_ROOT
 
     fake_tokenmap = json.dumps(
         {
@@ -971,7 +1147,7 @@ def _the_latest_complete_cluster_backup_is(context, expected_backup_name):
 @when(r"I truncate the backup index")
 def _truncate_the_index(context):
     storage = Storage(config=context.medusa_config.storage)
-    path_root = "/tmp/medusa_it_bucket"
+    path_root = BUCKET_ROOT
     index_path = "{}/{}index".format(path_root, storage.prefix_path)
     shutil.rmtree(index_path)
 
@@ -979,7 +1155,7 @@ def _truncate_the_index(context):
 @when(r"I truncate the backup folder")
 def _truncate_the_backup_folder(context):
     storage = Storage(config=context.medusa_config.storage)
-    path_root = "/tmp/medusa_it_bucket"
+    path_root = BUCKET_ROOT
     backup_path = "{}/{}127.0.0.1".format(path_root, storage.prefix_path)
     shutil.rmtree(backup_path)
 
@@ -1073,7 +1249,7 @@ def _i_can_see_secondary_index_files_in_backup(context, backup_name):
 @then(r'verify fails on the backup named "{backup_name}"')
 def _verify_fails_on_the_backup_named(context, backup_name):
     try:
-        medusa.verify.verify(context.medusa_config, backup_name)
+        medusa.verify.verify(context.medusa_config, backup_name, True)  # enable hash comparison
         raise AssertionError("Backup verification should have failed but didn't.")
     except RuntimeError:
         # This exception is required to be raised to validate the step
@@ -1175,3 +1351,65 @@ def write_dummy_file(path, mtime_str, contents=None):
     mtime = (t - datetime.datetime(1970, 1, 1)).total_seconds()
     atime = mtime
     os.utime(path, (atime, mtime))
+
+
+def get_mgmt_api_jars(
+        version,
+        url="https://api.github.com/repos/datastax/management-api-for-apache-cassandra/releases/latest"):
+
+    # clear out any temp resources that might exist
+    remove_temporary_mgmtapi_resources()
+
+    result = requests.get(url=url)
+
+    zip_file = requests.get(json.loads(result.text)["assets"][0]["browser_download_url"], stream=True)
+
+    with open("/tmp/mgmt_api_jars.zip", "wb") as mgmt_api_jars:
+        for chunk in zip_file.iter_content(chunk_size=4096):
+            if chunk:
+                mgmt_api_jars.write(chunk)
+
+    with ZipFile('/tmp/mgmt_api_jars.zip', 'r') as zip_ref:
+        list_of_names = zip_ref.namelist()
+        for file_name in list_of_names:
+            if file_name.endswith('.jar'):
+                if 'mgmtapi-agent' in file_name or 'mgmtapi-server' in file_name:
+                    zip_ref.extract(file_name, '/tmp')
+
+    symlink_mgmt_api_jar(version)
+
+
+def symlink_mgmt_api_jar(version):
+    management_api_jar_path = '/tmp/management-api-agent/target/datastax-mgmtapi-agent-0.1.0-SNAPSHOT.jar'
+    if not Path(management_api_jar_path).is_file():
+        # the bundle has split agents (maybe), one for C* 3.x and one for C* 4.x
+        # link the C* specific agent to the generic agent file name
+        assert False is Path('/tmp/management-api-agent/target/').exists()
+        Path('/tmp/management-api-agent/target/').mkdir(parents=True, exist_ok=False)
+        if str.startswith(version, '3'):
+            # C* is version 3.x, use 3.x agent
+            assert Path('/tmp/management-api-agent-3.x/target/datastax-mgmtapi-agent-3.x-0.1.0-SNAPSHOT.jar').is_file()
+            Path(management_api_jar_path).symlink_to(
+                '/tmp/management-api-agent-3.x/target/datastax-mgmtapi-agent-3.x-0.1.0-SNAPSHOT.jar')
+        elif str.startswith(version, '4') or 'github:apache/trunk' == version:
+            # C* is version 4.x, use 4.x agent
+            assert Path('/tmp/management-api-agent-4.x/target/datastax-mgmtapi-agent-4.x-0.1.0-SNAPSHOT.jar').is_file()
+            Path(management_api_jar_path).symlink_to(
+                '/tmp/management-api-agent-4.x/target/datastax-mgmtapi-agent-4.x-0.1.0-SNAPSHOT.jar')
+        else:
+            raise NotImplementedError('Cassandra version not supported: {}'.format(version))
+
+
+def rm_tree(pth):
+    pth = Path(pth)
+    for child in pth.glob('*'):
+        if child.is_file():
+            child.unlink()
+        else:
+            rm_tree(child)
+    pth.rmdir()
+
+
+def remove_temporary_mgmtapi_resources():
+    for tempDir in Path('/tmp').glob('management-api-*'):
+        rm_tree(tempDir)
